#include "RandTree.h"
#include <stdlib.h>
#include <math.h>
#include <stack>

#ifndef PI
#define PI 3.14159265
#endif


using namespace std;


void RandTree::generate() {
	while( iterations < 4 )
		step();
	
	clearThickness();
	calcThickness(thicknessInit);
}


/**
 * Go out to the ends of the branches (numChildren == 0) and add 2 random branches
 */
void RandTree::step() {
	int length;
	stack< Tree* > explore;
	
	
	if( root->numChildren == 0 ) {
		root->setNumChildren(1);
		root->children[0] = new Tree(0, 100, 0);
		
		iterations++;
		return;
	}
	
	length = 100 - iterations*9;
	if( length < 10 )
		length = 10;
	
	explore.push(root);
	
	while( !explore.empty() ) {
		Tree * current = explore.top();
		explore.pop();
		
		//Get the parent reference
		Tree * parent = NULL;
		if( !explore.empty() ) {
			parent = explore.top();
			explore.pop();
		}
		
		if( current->numChildren == 0 ) {
			//Leaf node
			float angle1 = drand48() * PI*3/4 - PI*3/8;
			float angle2 = drand48() * PI*3/4 - PI*3/8;
			float zoff1 = drand48() * length/2 - length/4;
			float zoff2 = drand48() * length/2 - length/4;
			
			//Find parent angle
			float angle = 0;
			if( parent != NULL ) {
				float dx = current->x - parent->x;
				float dy = current->y - parent->y;
				angle = atan2(dy,dx);
			}
			
			current->setNumChildren(2);
			current->children[0] = new Tree(current->x + length * cos(angle + angle1), 
											current->y + length * sin(angle + angle1), 
											current->z + zoff1 );
			current->children[1] = new Tree(current->x + length * cos(angle + angle2), 
											current->y + length * sin(angle + angle2), 
											current->z + zoff2 );
			
			//If stepping vs. generating, thickness won't be calculated, so set to 1 to be visible
			current->children[0]->thickness = 1;
			current->children[1]->thickness = 1;
			
		} else {
			//Branching node, push all children
			for(int i = 0; i < current->numChildren; i++) {
				explore.push(current);	//need reference to parent
				explore.push(current->children[i]);
			}
		}
		
	}
	
	iterations++;
}

