#include "Treegen.h"
#include <stdlib.h>
#include <math.h>
#include <stack>

#ifndef PI
#define PI 3.14159265
#endif

//Define actual variable here
int thicknessN = 3;		//good value for this is 3.  Can use 2 to make base wider for low-recursion trees.
int thicknessInit = 1;	//thickness at twig ends

using namespace std;



/**
 * Set tree branch thickness using Murry's method:
 * thickness = nth root( child1_thickness ^ N + child2_thickness ^ N)
 * 
 * Need Post-order depth-first traversal.  Done via a stack.
 */
void Treegen::calcThickness(int twigThickness) {
	stack< Tree* > explore;
	
	
	explore.push(root);
	while( !explore.empty() ) {
		Tree * current = explore.top();
		if( current->numChildren == 0 ) {
			//Leaf node -- get's twig thickness
			
			//Hmm, let it contribute and you get the narrowing effect on branches
			//Also, if it's zero, the exploration loop breaks because it assumes we don't set things to zero
			//if( current->bud )
			//	current->thickness = 0;	//buds don't contribute to branch thickness
			//else
				current->thickness = twigThickness;
			
			explore.pop();
		} else {
			//Branching node, first: push all children, second: calc thickness
			
			if( current->children[0]->thickness != 0 ) {
				//Already has thickness, so calculate ours
				explore.pop();	//and we're done with self
				
				if( current->numChildren == 1 ) {
					//Only one child, copy thickness
					current->thickness = current->children[0]->thickness;
				} else {
				
					//Find thickest two children
					Tree * child1 = NULL, * child2 = NULL;
					for(int i = 0; i < current->numChildren; i++) {
						int thk = current->children[i]->thickness;
						if( child1 == NULL || thk > child1->thickness ) {
							child2 = child1;	//pass down to second position (no harm if NULL)
							child1 = current->children[i];
						} else if( child2 == NULL || thk > child2->thickness ) {
							//Beats out second place
							child2 = current->children[i];
						}
					}
					
					current->thickness = pow( pow(child1->thickness, thicknessN) + 
											 pow(child2->thickness, thicknessN), 1.0 / thicknessN );
				}
			} else {
				//No thickness yet, so calc for children
				for(int i = 0; i < current->numChildren; i++) {
					explore.push(current->children[i]);
				}
			}
		}
		
	}
}

/**
 * Clear thickness back to 0 for calcThickness can work
 */
void Treegen::clearThickness() {
	stack< Tree* > explore;
	
	
	explore.push(root);
	while( !explore.empty() ) {
		Tree * current = explore.top();
		explore.pop();
		
		current->thickness = 0;
		
		for(int i = 0; i < current->numChildren; i++) {
			explore.push(current->children[i]);
		}
	}
}

