#pragma once

#include "Vector.h"

//Index into the array for 3D coordinates
#define I(x,y,z)  (((int)(z)) * height*width + ((int)(y))*width + ((int)(x)))

//Settings - used in casting shadow/shelter
extern float shadowBase;		//a
extern float shadowFactor;		//b
extern float lightMax;			//C
extern int qMax;


class VoxelGrid
{
	private:
	//Data
	float * grid;
	int width;
	int height;
	int depth;
	
	public:
	
	
	
	
	/**
	 * Constructor
	 */
	VoxelGrid(int _width, int _height, int _depth)
	: grid(NULL)
	{
		width = _width;
		height = _height;
		depth = _depth;
		grid = new float[width * height * depth];
		
		//Initialize to zero (not doing memset because I don't know if 0 is 0 in float...)
		for(int z = 0; z < depth; z++) {
			for(int y = 0; y < height; y++) {
				for(int x = 0; x < width; x++) {
					grid[I(x,y,z)] = 0;
				}
			}
		}
	}
	
	
	/**
	 * Destructor
	 */
	~VoxelGrid() {
		delete grid;
	}
	
	
	float get(int x, int y, int z) {
		return grid[I(x, y, z)];
	}
	int getWidth() { return width; }
	int getHeight() { return height; }
	int getDepth() { return depth; }
	
	Vec3i pixelsToVoxel(float x, float y, float z, int pixPerVox);
	Vec3 voxelToPixels(int x, int y, int z, int pixPerVox);
	Vec3i *getNeighbors(int x, int y, int z);
	float exposure(int x, int y, int z);
	
	void occlude(Vec3i loc, Vec3 dir);
	void occludeDown(Vec3i loc);
	void occludeRight(Vec3i loc);
	
};
