#include "WindTree.h"
#include <stdlib.h>
#include <math.h>
#include <stack>
#include <algorithm>
#include "DrawVoxels.h"

#ifndef PI
#define PI 3.14159265
#endif


//Since we use the same exposure calc as light/shadow, this factor will scale it if wind
//needs different values
//#define WIND_FACTOR 1.0
float WIND_FACTOR = 1.0/8.0;

//If wind should affect growth direction of terminal buds, bending the branches (light doesn't)
int windBendsBranches = 1;

using namespace std;


/**
 * Draws debug info
 */
void WindTree::drawInfo() {
	//draw_voxels(voxels, PIXELS_PER_VOXEL);
	draw_voxels(windVoxels, WIND_PIXELS_PER_VOXEL);
}



/**
 * Determines growth direction for all buds.
 * First let LightTree figure out directions.  Then use the wind voxel grid to
 * bend them away from the wind.
 */
void WindTree::calcVectors() {
	//Calc from light exposure
	LightTree::calcVectors();
	
	//Now bend with wind
	vector<Tree*>::const_iterator it;
	for(it = buds.begin(); it != buds.end(); it++) {
		Tree *bud = *it;
		
		if( !windBendsBranches && bud->branch == 0 )
			continue;	//terminal bud, don't change growth direction
		
		double wind = calcWind(bud);
		Vec3 windv = Vec3(wind, 0, 0);
		bud->growth += windv;
		bud->growth.Normalize();
		
	}//continue to next bud
}



/**
 * Find how much wind exposure this bud has
 */
double WindTree::calcWind(Tree *bud) {
	//Find what voxel bud is in
	Vec3i v = windVoxels->pixelsToVoxel(bud->x, bud->y, bud->z, WIND_PIXELS_PER_VOXEL);
	//Get shelter at that voxel
	return WIND_FACTOR * windVoxels->exposure(v.x, v.y, v.z);
}

/**
 * Cast shelter sideways and shadow down.
 */
void WindTree::occlude(Tree *bud) {
	LightTree::occlude(bud);
	
	Vec3i vox = windVoxels->pixelsToVoxel(bud->x, bud->y, bud->z, WIND_PIXELS_PER_VOXEL);
	windVoxels->occludeRight(vox);
}

