#include "Writer.h"
#include <stdio.h>
#include <stack>

typedef struct _TreeState {
	Tree * tree;
	int nextChild;
} TreeState;


Tree* loadNode(FILE *file) {
	Tree * node;
	float x, y, z, thickness;
	int children;
	char buf[20];
	buf[19] = '\0';	//do I need this???  I think the space would stop atof, atoi.
	
	//printf("Reading node...\n");
	
	//Read in values
	fread((void*)buf, sizeof(char), 19, file);	//18 chars number, 1 char space
	x = (float)atof(buf);
	//printf("read x=%18.10f\n",x);
	
	fread((void*)buf, sizeof(char), 19, file);	//18 chars number, 1 char space
	y = (float)atof(buf);
	//printf("read y=%18.10f\n",y);
	
	fread((void*)buf, sizeof(char), 19, file);	//18 chars number, 1 char space
	z = (float)atof(buf);
	//printf("read z=%18.10f\n",z);
	
	fread((void*)buf, sizeof(char), 16, file);	//15 chars number, 1 char space
	thickness = (float)atof(buf);
	//printf("read thickness=%15.10f\n",thickness);
	
	fread((void*)buf, sizeof(char), 3, file);	//3 chars number
	buf[3] = '\0';	//didn't read a space, so need some separator
	children = atoi(buf);
	//printf("read children=%d\n",children);
	
	buf[0] = getc(file);	//read the newline char
	if( buf[0] != '\n' ) {
		printf("Error! Expected newline");
		return NULL;
	}
	
	node = new Tree(x,y,z);
	node->thickness = thickness;
	node->setNumChildren(children);
	
	//printf("...read node (%5.3f,%5.3f,%5.3f)\n", x, y, z);
	
	return node;
}


Tree * loadFile(char * filename) {
	FILE * file;
	Tree * tree;
	TreeState node;
	std::stack< TreeState > explore;
	
	file = fopen(filename,"r");
	
	//printf("Reading first node\n");
	tree = loadNode(file);
	
	TreeState initial = { tree, 0 };
	explore.push(initial);
	while( !explore.empty() ) {
		node = explore.top();
		explore.pop();
		
		if( node.tree->numChildren == node.nextChild ) {
			//printf("Done loading children of (%5.3f, %5.3f, %5.3f)\n", node.tree->x, node.tree->y, node.tree->z);
			//done with this as a parent, so don't push back on
		} else {
			//Still need to read another child
			//printf("Reading child %d of (%5.3f, %5.3f, %5.3f)\n", node.nextChild, 
			//	   node.tree->x, node.tree->y, node.tree->z);
			
			Tree * child = loadNode(file);
			node.tree->children[node.nextChild] = child;
			
			node.nextChild++;	//will read the next child when we get back to this node
			explore.push(node);	//put back on with new value
			TreeState add = {child, 0};
			explore.push(add);	//need to read this child's children
		}
	}
	
	fclose(file);
	printf("Done\n");
	
	return tree;
}

void saveFile(char * filename, Tree * tree) {
	FILE * file;
	int i;
	std::stack< Tree* > explore;
	
	file = fopen(filename,"w+");
	
	//Write each node, then children
	explore.push(tree);
	
	while( !explore.empty() ) {
		Tree * current = explore.top();
		explore.pop();
		
		//Print node
		fprintf(file, "%18.10f %18.10f %18.10f %15.10f %3d\n",
				current->x, current->y, current->z, current->thickness, current->numChildren);
		
		//Print children
		for(i = 0; i < current->numChildren; i++) {
			explore.push(current->children[i]);
		}
		
	}
	
	fclose(file);
}
